# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_gamenotifications.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CGameNotificationsVariable(betterproto.Message):
    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CGameNotificationsLocalizedText(betterproto.Message):
    token: str = betterproto.string_field(1)
    variables: List["CGameNotificationsVariable"] = betterproto.message_field(2)
    rendered_text: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CGameNotificationsUserStatus(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    state: str = betterproto.string_field(2)
    title: "CGameNotificationsLocalizedText" = betterproto.message_field(3)
    message: "CGameNotificationsLocalizedText" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CGameNotificationsCreateSessionRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    context: int = betterproto.uint64_field(2)
    title: "CGameNotificationsLocalizedText" = betterproto.message_field(3)
    users: List["CGameNotificationsUserStatus"] = betterproto.message_field(4)
    steamid: int = betterproto.fixed64_field(5)


@dataclass(eq=False, repr=False)
class CGameNotificationsCreateSessionResponse(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CGameNotificationsDeleteSessionRequest(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint32_field(2)
    steamid: int = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CGameNotificationsDeleteSessionResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CGameNotificationsUpdateSessionRequest(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint32_field(2)
    title: "CGameNotificationsLocalizedText" = betterproto.message_field(3)
    users: List["CGameNotificationsUserStatus"] = betterproto.message_field(4)
    steamid: int = betterproto.fixed64_field(6)


@dataclass(eq=False, repr=False)
class CGameNotificationsUpdateSessionResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CGameNotificationsEnumerateSessionsRequest(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    include_all_user_messages: bool = betterproto.bool_field(3)
    include_auth_user_message: bool = betterproto.bool_field(4)
    language: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class CGameNotificationsSession(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint64_field(2)
    context: int = betterproto.uint64_field(3)
    title: "CGameNotificationsLocalizedText" = betterproto.message_field(4)
    time_created: int = betterproto.uint32_field(5)
    time_updated: int = betterproto.uint32_field(6)
    user_status: List["CGameNotificationsUserStatus"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class CGameNotificationsEnumerateSessionsResponse(betterproto.Message):
    sessions: List["CGameNotificationsSession"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CGameNotificationsGetSessionDetailsRequest(betterproto.Message):
    sessions: List["CGameNotificationsGetSessionDetailsRequestRequestedSession"] = betterproto.message_field(1)
    appid: int = betterproto.uint32_field(2)
    language: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CGameNotificationsGetSessionDetailsRequestRequestedSession(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)
    include_auth_user_message: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CGameNotificationsGetSessionDetailsResponse(betterproto.Message):
    sessions: List["CGameNotificationsSession"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GameNotificationSettings(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    allow_notifications: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CGameNotificationsUpdateNotificationSettingsRequest(betterproto.Message):
    game_notification_settings: List["GameNotificationSettings"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CGameNotificationsUpdateNotificationSettingsResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CGameNotificationsOnNotificationsRequestedNotification(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    appid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CGameNotificationsOnUserStatusChangedNotification(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    sessionid: int = betterproto.uint64_field(2)
    appid: int = betterproto.uint32_field(3)
    status: "CGameNotificationsUserStatus" = betterproto.message_field(4)
    removed: bool = betterproto.bool_field(5)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_gameservers.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CGameServersGetServerListRequest(betterproto.Message):
    filter: str = betterproto.string_field(1)
    limit: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CGameServersGetServerListResponse(betterproto.Message):
    servers: List["CGameServersGetServerListResponseServer"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CGameServersGetServerListResponseServer(betterproto.Message):
    addr: str = betterproto.string_field(1)
    gameport: int = betterproto.uint32_field(2)
    specport: int = betterproto.uint32_field(3)
    steamid: int = betterproto.fixed64_field(4)
    name: str = betterproto.string_field(5)
    appid: int = betterproto.uint32_field(6)
    gamedir: str = betterproto.string_field(7)
    version: str = betterproto.string_field(8)
    product: str = betterproto.string_field(9)
    region: int = betterproto.int32_field(10)
    players: int = betterproto.int32_field(11)
    max_players: int = betterproto.int32_field(12)
    bots: int = betterproto.int32_field(13)
    map: str = betterproto.string_field(14)
    secure: bool = betterproto.bool_field(15)
    dedicated: bool = betterproto.bool_field(16)
    os: str = betterproto.string_field(17)
    gametype: str = betterproto.string_field(18)


@dataclass(eq=False, repr=False)
class CGameServersGetServerSteamIDsByIpRequest(betterproto.Message):
    server_ips: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CGameServersIPsWithSteamIDsResponse(betterproto.Message):
    servers: List["CGameServersIPsWithSteamIDsResponseServer"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CGameServersIPsWithSteamIDsResponseServer(betterproto.Message):
    addr: str = betterproto.string_field(1)
    steamid: int = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CGameServersGetServerIPsBySteamIdRequest(betterproto.Message):
    server_steamids: List[int] = betterproto.fixed64_field(1)



from . import (
    steammessages_chat,
    steammessages_econ,
    steammessages_friendmessages,
    steammessages_gameservers,
    steammessages_player,
    steammessages_twofactor,
    steammessages_useraccount,
)

__all__ = ("UMS",)


UMS = {
    "GameServers.GetServerList#1_Request": steammessages_gameservers.CGameServersGetServerListRequest,
    "GameServers.GetServerList#1_Response": steammessages_gameservers.CGameServersGetServerListResponse,
    "GameServers.GetServerSteamIDsByIP#1_Request": steammessages_gameservers.CGameServersGetServerSteamIDsByIpRequest,
    "GameServers.GetServerSteamIDsByIP#1_Response": steammessages_gameservers.CGameServersIPsWithSteamIDsResponse,
    "GameServers.GetServerIPsBySteamID#1_Request": steammessages_gameservers.CGameServersGetServerIPsBySteamIdRequest,
    "GameServers.GetServerIPsBySteamID#1_Response": steammessages_gameservers.CGameServersIPsWithSteamIDsResponse,
    "TwoFactor.AddAuthenticator#1_Request": steammessages_twofactor.CTwoFactorAddAuthenticatorRequest,
    "TwoFactor.AddAuthenticator#1_Response": steammessages_twofactor.CTwoFactorAddAuthenticatorResponse,
    "TwoFactor.FinalizeAddAuthenticator#1_Request": steammessages_twofactor.CTwoFactorFinalizeAddAuthenticatorRequest,
    "TwoFactor.FinalizeAddAuthenticator#1_Response": steammessages_twofactor.CTwoFactorFinalizeAddAuthenticatorResponse,
    "TwoFactor.SendEmail#1_Request": steammessages_twofactor.CTwoFactorSendEmailRequest,
    "TwoFactor.SendEmail#1_Response": steammessages_twofactor.CTwoFactorSendEmailResponse,
    "TwoFactor.RemoveAuthenticator#1_Request": steammessages_twofactor.CTwoFactorRemoveAuthenticatorRequest,
    "TwoFactor.RemoveAuthenticator#1_Response": steammessages_twofactor.CTwoFactorRemoveAuthenticatorResponse,
    "Player.GetGameBadgeLevels#1_Request": steammessages_player.CPlayerGetGameBadgeLevelsRequest,
    "Player.GetGameBadgeLevels#1_Response": steammessages_player.CPlayerGetGameBadgeLevelsResponse,
    "Player.GetNicknameList#1_Request": steammessages_player.CPlayerGetNicknameListRequest,
    "Player.GetNicknameList#1_Response": steammessages_player.CPlayerGetNicknameListResponse,
    "Player.GetEmoticonList#1_Request": steammessages_player.CPlayerGetEmoticonListRequest,
    "Player.GetEmoticonList#1_Response": steammessages_player.CPlayerGetEmoticonListResponse,
    "Player.GetPrivacySettings#1_Request": steammessages_player.CPlayerGetPrivacySettingsRequest,
    "Player.GetPrivacySettings#1_Response": steammessages_player.CPlayerGetPrivacySettingsResponse,
    "Player.GetOwnedGames#1_Request": steammessages_player.CPlayerGetOwnedGamesRequest,
    "Player.GetOwnedGames#1_Response": steammessages_player.CPlayerGetOwnedGamesResponse,
    "Econ.GetAssetClassInfo#1_Request": steammessages_econ.CEconGetAssetClassInfoRequest,
    "Econ.GetAssetClassInfo#1_Response": steammessages_econ.CEconGetAssetClassInfoResponse,
    "Econ.GetTradeOfferAccessToken#1_Request": steammessages_econ.CEconGetTradeOfferAccessTokenRequest,
    "Econ.GetTradeOfferAccessToken#1_Response": steammessages_econ.CEconGetTradeOfferAccessTokenResponse,
    "ChatRoom.CreateChatRoomGroup#1_Request": steammessages_chat.CChatRoomCreateChatRoomGroupRequest,
    "ChatRoom.CreateChatRoomGroup#1_Response": steammessages_chat.CChatRoomCreateChatRoomGroupResponse,
    "ChatRoom.SaveChatRoomGroup#1_Request": steammessages_chat.CChatRoomSaveChatRoomGroupRequest,
    "ChatRoom.SaveChatRoomGroup#1_Response": steammessages_chat.CChatRoomSaveChatRoomGroupResponse,
    "ChatRoom.RenameChatRoomGroup#1_Request": steammessages_chat.CChatRoomRenameChatRoomGroupRequest,
    "ChatRoom.RenameChatRoomGroup#1_Response": steammessages_chat.CChatRoomRenameChatRoomGroupResponse,
    "ChatRoom.SetChatRoomGroupTagline#1_Request": steammessages_chat.CChatRoomSetChatRoomGroupTaglineRequest,
    "ChatRoom.SetChatRoomGroupTagline#1_Response": steammessages_chat.CChatRoomSetChatRoomGroupTaglineResponse,
    "ChatRoom.SetChatRoomGroupAvatar#1_Request": steammessages_chat.CChatRoomSetChatRoomGroupAvatarRequest,
    "ChatRoom.SetChatRoomGroupAvatar#1_Response": steammessages_chat.CChatRoomSetChatRoomGroupAvatarResponse,
    "ChatRoom.MuteUserInGroup#1_Request": steammessages_chat.CChatRoomMuteUserRequest,
    "ChatRoom.MuteUserInGroup#1_Response": steammessages_chat.CChatRoomMuteUserResponse,
    "ChatRoom.KickUserFromGroup#1_Request": steammessages_chat.CChatRoomKickUserRequest,
    "ChatRoom.KickUserFromGroup#1_Response": steammessages_chat.CChatRoomKickUserResponse,
    "ChatRoom.SetUserBanState#1_Request": steammessages_chat.CChatRoomSetUserBanStateRequest,
    "ChatRoom.SetUserBanState#1_Response": steammessages_chat.CChatRoomSetUserBanStateResponse,
    "ChatRoom.RevokeInviteToGroup#1_Request": steammessages_chat.CChatRoomRevokeInviteRequest,
    "ChatRoom.RevokeInviteToGroup#1_Response": steammessages_chat.CChatRoomRevokeInviteResponse,
    "ChatRoom.CreateRole#1_Request": steammessages_chat.CChatRoomCreateRoleRequest,
    "ChatRoom.CreateRole#1_Response": steammessages_chat.CChatRoomCreateRoleResponse,
    "ChatRoom.GetRoles#1_Request": steammessages_chat.CChatRoomGetRolesRequest,
    "ChatRoom.GetRoles#1_Response": steammessages_chat.CChatRoomGetRolesResponse,
    "ChatRoom.RenameRole#1_Request": steammessages_chat.CChatRoomRenameRoleRequest,
    "ChatRoom.RenameRole#1_Response": steammessages_chat.CChatRoomRenameRoleResponse,
    "ChatRoom.ReorderRole#1_Request": steammessages_chat.CChatRoomReorderRoleRequest,
    "ChatRoom.ReorderRole#1_Response": steammessages_chat.CChatRoomReorderRoleResponse,
    "ChatRoom.DeleteRole#1_Request": steammessages_chat.CChatRoomDeleteRoleRequest,
    "ChatRoom.DeleteRole#1_Response": steammessages_chat.CChatRoomDeleteRoleResponse,
    "ChatRoom.GetRoleActions#1_Request": steammessages_chat.CChatRoomGetRoleActionsRequest,
    "ChatRoom.GetRoleActions#1_Response": steammessages_chat.CChatRoomGetRoleActionsResponse,
    "ChatRoom.ReplaceRoleActions#1_Request": steammessages_chat.CChatRoomReplaceRoleActionsRequest,
    "ChatRoom.ReplaceRoleActions#1_Response": steammessages_chat.CChatRoomReplaceRoleActionsResponse,
    "ChatRoom.AddRoleToUser#1_Request": steammessages_chat.CChatRoomAddRoleToUserRequest,
    "ChatRoom.AddRoleToUser#1_Response": steammessages_chat.CChatRoomAddRoleToUserResponse,
    "ChatRoom.GetRolesForUser#1_Request": steammessages_chat.CChatRoomGetRolesForUserRequest,
    "ChatRoom.GetRolesForUser#1_Response": steammessages_chat.CChatRoomGetRolesForUserResponse,
    "ChatRoom.DeleteRoleFromUser#1_Request": steammessages_chat.CChatRoomDeleteRoleFromUserRequest,
    "ChatRoom.DeleteRoleFromUser#1_Response": steammessages_chat.CChatRoomDeleteRoleFromUserResponse,
    "ChatRoom.JoinChatRoomGroup#1_Request": steammessages_chat.CChatRoomJoinChatRoomGroupRequest,
    "ChatRoom.JoinChatRoomGroup#1_Response": steammessages_chat.CChatRoomJoinChatRoomGroupResponse,
    "ChatRoom.InviteFriendToChatRoomGroup#1_Request": steammessages_chat.CChatRoomInviteFriendToChatRoomGroupRequest,
    "ChatRoom.InviteFriendToChatRoomGroup#1_Response": steammessages_chat.CChatRoomInviteFriendToChatRoomGroupResponse,
    "ChatRoom.LeaveChatRoomGroup#1_Request": steammessages_chat.CChatRoomLeaveChatRoomGroupRequest,
    "ChatRoom.LeaveChatRoomGroup#1_Response": steammessages_chat.CChatRoomLeaveChatRoomGroupResponse,
    "ChatRoom.CreateChatRoom#1_Request": steammessages_chat.CChatRoomCreateChatRoomRequest,
    "ChatRoom.CreateChatRoom#1_Response": steammessages_chat.CChatRoomCreateChatRoomResponse,
    "ChatRoom.DeleteChatRoom#1_Request": steammessages_chat.CChatRoomDeleteChatRoomRequest,
    "ChatRoom.DeleteChatRoom#1_Response": steammessages_chat.CChatRoomDeleteChatRoomResponse,
    "ChatRoom.RenameChatRoom#1_Request": steammessages_chat.CChatRoomRenameChatRoomRequest,
    "ChatRoom.RenameChatRoom#1_Response": steammessages_chat.CChatRoomRenameChatRoomResponse,
    "ChatRoom.SendChatMessage#1_Request": steammessages_chat.CChatRoomSendChatMessageRequest,
    "ChatRoom.SendChatMessage#1_Response": steammessages_chat.CChatRoomSendChatMessageResponse,
    "ChatRoom.JoinVoiceChat#1_Request": steammessages_chat.CChatRoomJoinVoiceChatRequest,
    "ChatRoom.JoinVoiceChat#1_Response": steammessages_chat.CChatRoomJoinVoiceChatResponse,
    "ChatRoom.LeaveVoiceChat#1_Request": steammessages_chat.CChatRoomLeaveVoiceChatRequest,
    "ChatRoom.LeaveVoiceChat#1_Response": steammessages_chat.CChatRoomLeaveVoiceChatResponse,
    "ChatRoom.GetMessageHistory#1_Request": steammessages_chat.CChatRoomGetMessageHistoryRequest,
    "ChatRoom.GetMessageHistory#1_Response": steammessages_chat.CChatRoomGetMessageHistoryResponse,
    "ChatRoom.GetMyChatRoomGroups#1_Request": steammessages_chat.CChatRoomGetMyChatRoomGroupsRequest,
    "ChatRoom.GetMyChatRoomGroups#1_Response": steammessages_chat.CChatRoomGetMyChatRoomGroupsResponse,
    "ChatRoom.GetChatRoomGroupState#1_Request": steammessages_chat.CChatRoomGetChatRoomGroupStateRequest,
    "ChatRoom.GetChatRoomGroupState#1_Response": steammessages_chat.CChatRoomGetChatRoomGroupStateResponse,
    "ChatRoom.GetChatRoomGroupSummary#1_Request": steammessages_chat.CChatRoomGetChatRoomGroupSummaryRequest,
    "ChatRoom.GetChatRoomGroupSummary#1_Response": steammessages_chat.CChatRoomGetChatRoomGroupSummaryResponse,
    "ChatRoom.AckChatMessage#1_Request": steammessages_chat.CChatRoomAckChatMessageNotification,
    "ChatRoom.CreateInviteLink#1_Request": steammessages_chat.CChatRoomCreateInviteLinkRequest,
    "ChatRoom.CreateInviteLink#1_Response": steammessages_chat.CChatRoomCreateInviteLinkResponse,
    "ChatRoom.GetInviteLinkInfo#1_Request": steammessages_chat.CChatRoomGetInviteLinkInfoRequest,
    "ChatRoom.GetInviteLinkInfo#1_Response": steammessages_chat.CChatRoomGetInviteLinkInfoResponse,
    "ChatRoom.GetInviteInfo#1_Request": steammessages_chat.CChatRoomGetInviteInfoRequest,
    "ChatRoom.GetInviteInfo#1_Response": steammessages_chat.CChatRoomGetInviteInfoResponse,
    "ChatRoom.GetInviteLinksForGroup#1_Request": steammessages_chat.CChatRoomGetInviteLinksForGroupRequest,
    "ChatRoom.GetInviteLinksForGroup#1_Response": steammessages_chat.CChatRoomGetInviteLinksForGroupResponse,
    "ChatRoom.GetBanList#1_Request": steammessages_chat.CChatRoomGetBanListRequest,
    "ChatRoom.GetBanList#1_Response": steammessages_chat.CChatRoomGetBanListResponse,
    "ChatRoom.GetInviteList#1_Request": steammessages_chat.CChatRoomGetInviteListRequest,
    "ChatRoom.GetInviteList#1_Response": steammessages_chat.CChatRoomGetInviteListResponse,
    "ChatRoom.DeleteInviteLink#1_Request": steammessages_chat.CChatRoomDeleteInviteLinkRequest,
    "ChatRoom.DeleteInviteLink#1_Response": steammessages_chat.CChatRoomDeleteInviteLinkResponse,
    "ChatRoom.SetSessionActiveChatRoomGroups#1_Request": steammessages_chat.CChatRoomSetSessionActiveChatRoomGroupsRequest,
    "ChatRoom.SetSessionActiveChatRoomGroups#1_Response": steammessages_chat.CChatRoomSetSessionActiveChatRoomGroupsResponse,
    "ChatRoom.SetUserChatGroupPreferences#1_Request": steammessages_chat.CChatRoomSetUserChatGroupPreferencesRequest,
    "ChatRoom.SetUserChatGroupPreferences#1_Response": steammessages_chat.CChatRoomSetUserChatGroupPreferencesResponse,
    "ChatRoom.DeleteChatMessages#1_Request": steammessages_chat.CChatRoomDeleteChatMessagesRequest,
    "ChatRoom.DeleteChatMessages#1_Response": steammessages_chat.CChatRoomDeleteChatMessagesResponse,
    "ChatRoom.NotifyMessageReaction#1_Response": steammessages_chat.CChatRoomMessageReactionNotification,
    "ChatRoomClient.NotifyIncomingChatMessage#1_Request": steammessages_chat.CChatRoomIncomingChatMessageNotification,
    "ChatRoomClient.NotifyChatMessageModified#1_Request": steammessages_chat.CChatRoomChatMessageModifiedNotification,
    "ChatRoomClient.NotifyMemberStateChange#1_Request": steammessages_chat.CChatRoomMemberStateChangeNotification,
    "ChatRoomClient.NotifyChatRoomHeaderStateChange#1_Request": steammessages_chat.CChatRoomChatRoomHeaderStateNotification,
    "ChatRoomClient.NotifyChatRoomGroupRoomsChange#1_Request": steammessages_chat.CChatRoomChatRoomGroupRoomsChangeNotification,
    "ChatRoomClient.NotifyShouldRejoinChatRoomVoiceChat#1_Request": steammessages_chat.CChatRoomNotifyShouldRejoinChatRoomVoiceChatNotification,
    "ChatRoomClient.NotifyChatGroupUserStateChanged#1_Request": steammessages_chat.ChatRoomClientNotifyChatGroupUserStateChangedNotification,
    "ChatRoomClient.NotifyAckChatMessageEcho#1_Request": steammessages_chat.CChatRoomAckChatMessageNotification,
    "ClanChatRooms.GetClanChatRoomInfo#1_Request": steammessages_chat.CClanChatRoomsGetClanChatRoomInfoRequest,
    "ClanChatRooms.GetClanChatRoomInfo#1_Response": steammessages_chat.CClanChatRoomsGetClanChatRoomInfoResponse,
    "FriendMessages.GetActiveMessageSessions#1_Request": steammessages_friendmessages.CFriendsMessagesGetActiveMessageSessionsRequest,
    "FriendMessages.GetActiveMessageSessions#1_Response": steammessages_friendmessages.CFriendsMessagesGetActiveMessageSessionsResponse,
    "FriendMessages.SendMessage#1_Request": steammessages_friendmessages.CFriendMessagesSendMessageRequest,
    "FriendMessages.SendMessage#1_Response": steammessages_friendmessages.CFriendMessagesSendMessageResponse,
    "FriendMessages.AckMessage#1_Request": steammessages_friendmessages.CFriendMessagesAckMessageNotification,
    "FriendMessages.GetRecentMessages#1_Request": steammessages_friendmessages.CFriendMessagesGetRecentMessagesRequest,
    "FriendMessages.GetRecentMessages#1_Response": steammessages_friendmessages.CFriendMessagesGetRecentMessagesResponse,
    "FriendMessages.UpdateMessageReaction#1_Request": steammessages_friendmessages.CFriendMessagesUpdateMessageReactionRequest,
    "FriendMessages.UpdateMessageReaction#1_Response": steammessages_friendmessages.CFriendMessagesUpdateMessageReactionResponse,
    "FriendMessagesClient.IncomingMessage#1_Request": steammessages_friendmessages.CFriendMessagesIncomingMessageNotification,
    "FriendMessagesClient.NotifyAckMessageEcho#1_Request": steammessages_friendmessages.CFriendMessagesAckMessageNotification,
    "FriendMessagesClient.MessageReaction#1_Request": steammessages_friendmessages.CFriendMessagesMessageReactionNotification,
    "UserAccount.CreateFriendInviteToken#1_Request": steammessages_useraccount.CUserAccountCreateFriendInviteTokenRequest,
    "UserAccount.CreateFriendInviteToken#1_Response": steammessages_useraccount.CUserAccountCreateFriendInviteTokenResponse,
    "UserAccount.GetFriendInviteTokens#1_Request": steammessages_useraccount.CUserAccountGetFriendInviteTokensRequest,
    "UserAccount.GetFriendInviteTokens#1_Response": steammessages_useraccount.CUserAccountGetFriendInviteTokensResponse,
    "UserAccount.ViewFriendInviteToken#1_Request": steammessages_useraccount.CUserAccountViewFriendInviteTokenRequest,
    "UserAccount.ViewFriendInviteToken#1_Response": steammessages_useraccount.CUserAccountViewFriendInviteTokenResponse,
    "UserAccount.RedeemFriendInviteToken#1_Request": steammessages_useraccount.CUserAccountRedeemFriendInviteTokenRequest,
    "UserAccount.RedeemFriendInviteToken#1_Response": steammessages_useraccount.CUserAccountRedeemFriendInviteTokenResponse,
    "UserAccount.RevokeFriendInviteToken#1_Request": steammessages_useraccount.CUserAccountRevokeFriendInviteTokenRequest,
}



import platform

import aiohttp
import betterproto

import steam


def main() -> None:
    print("python version:", platform.python_version())
    print("steam.py version:", steam.__version__)
    print("aiohttp version:", aiohttp.__version__)
    print("betterproto version:", betterproto.__version__)
    print("operating system info:", platform.platform())


if __name__ == "__main__":
    main()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_chat.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto

from .steammessages_clientserver_friends import CMsgClientPersonaStateFriend


class EChatRoomJoinState(betterproto.Enum):
    Default = 0
    NONE = 1
    Joined = 2
    TestInvalid = 99


class EChatRoomGroupRank(betterproto.Enum):
    Default = 0
    Viewer = 10
    Guest = 15
    Member = 20
    Moderator = 30
    Officer = 40
    Owner = 50
    TestInvalid = 99


class EChatRoomNotificationLevel(betterproto.Enum):
    Invalid = 0
    NONE = 1
    MentionMe = 2
    MentionAll = 3
    AllMessages = 4


class EChatRoomServerMessage(betterproto.Enum):
    Invalid = 0
    RenameChatRoom = 1
    Joined = 2
    Parted = 3
    Kicked = 4
    Invited = 5
    InviteDismissed = 8
    ChatRoomTaglineChanged = 9
    ChatRoomAvatarChanged = 10
    AppCustom = 11


class EChatRoomMessageReactionType(betterproto.Enum):
    Invalid = 0
    Emoticon = 1


class EChatRoomMemberStateChange(betterproto.Enum):
    Invalid = 0
    Joined = 1
    Parted = 2
    Kicked = 3
    Invited = 4
    RankChanged = 7
    InviteDismissed = 8
    Muted = 9
    Banned = 10
    RolesChanged = 12


@dataclass(eq=False, repr=False)
class CChatRequestFriendPersonaStatesRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRequestFriendPersonaStatesResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomCreateChatRoomGroupRequest(betterproto.Message):
    steamid_partner: int = betterproto.fixed64_field(1)
    steamid_invited: int = betterproto.fixed64_field(2)
    name: str = betterproto.string_field(3)
    steamid_invitees: List[int] = betterproto.fixed64_field(4)
    watching_broadcast_accountid: int = betterproto.uint32_field(6)
    watching_broadcast_channel_id: int = betterproto.uint64_field(7)


@dataclass(eq=False, repr=False)
class CChatRole(betterproto.Message):
    role_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)
    ordinal: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoleActions(betterproto.Message):
    role_id: int = betterproto.uint64_field(1)
    can_create_rename_delete_channel: bool = betterproto.bool_field(2)
    can_kick: bool = betterproto.bool_field(3)
    can_ban: bool = betterproto.bool_field(4)
    can_invite: bool = betterproto.bool_field(5)
    can_change_tagline_avatar_name: bool = betterproto.bool_field(6)
    can_chat: bool = betterproto.bool_field(7)
    can_view_history: bool = betterproto.bool_field(8)
    can_change_group_roles: bool = betterproto.bool_field(9)
    can_change_user_roles: bool = betterproto.bool_field(10)
    can_mention_all: bool = betterproto.bool_field(11)
    can_set_watching_broadcast: bool = betterproto.bool_field(12)


@dataclass(eq=False, repr=False)
class CChatPartyBeacon(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    steamid_owner: int = betterproto.fixed64_field(2)
    beacon_id: int = betterproto.fixed64_field(3)
    game_metadata: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGroupHeaderState(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_name: str = betterproto.string_field(2)
    clanid: int = betterproto.uint32_field(13)
    accountid_owner: int = betterproto.uint32_field(14)
    appid: int = betterproto.uint32_field(21)
    tagline: str = betterproto.string_field(15)
    avatar_sha: bytes = betterproto.bytes_field(16)
    default_role_id: int = betterproto.uint64_field(17)
    roles: List["CChatRole"] = betterproto.message_field(18)
    role_actions: List["CChatRoleActions"] = betterproto.message_field(19)
    watching_broadcast_accountid: int = betterproto.uint32_field(20)
    party_beacons: List["CChatPartyBeacon"] = betterproto.message_field(22)
    watching_broadcast_channel_id: int = betterproto.uint64_field(23)
    active_minigame_id: int = betterproto.uint64_field(24)
    avatar_ugc_url: str = betterproto.string_field(25)


@dataclass(eq=False, repr=False)
class CChatRoomMember(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    state: "EChatRoomJoinState" = betterproto.enum_field(3)
    rank: "EChatRoomGroupRank" = betterproto.enum_field(4)
    time_kick_expire: int = betterproto.uint32_field(6)
    role_ids: List[int] = betterproto.uint64_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomState(betterproto.Message):
    chat_id: int = betterproto.uint64_field(1)
    chat_name: str = betterproto.string_field(2)
    voice_allowed: bool = betterproto.bool_field(3)
    members_in_voice: List[int] = betterproto.uint32_field(4)
    time_last_message: int = betterproto.uint32_field(5)
    sort_order: int = betterproto.uint32_field(6)
    last_message: str = betterproto.string_field(7)
    accountid_last_message: int = betterproto.uint32_field(8)


@dataclass(eq=False, repr=False)
class CChatRoomGroupState(betterproto.Message):
    header_state: "CChatRoomGroupHeaderState" = betterproto.message_field(1)
    members: List["CChatRoomMember"] = betterproto.message_field(2)
    default_chat_id: int = betterproto.uint64_field(4)
    chat_rooms: List["CChatRoomState"] = betterproto.message_field(5)
    kicked: List["CChatRoomMember"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class CUserChatRoomState(betterproto.Message):
    chat_id: int = betterproto.uint64_field(1)
    time_joined: int = betterproto.uint32_field(2)
    time_last_ack: int = betterproto.uint32_field(3)
    desktop_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(4)
    mobile_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(5)
    time_last_mention: int = betterproto.uint32_field(6)
    unread_indicator_muted: bool = betterproto.bool_field(7)
    time_first_unread: int = betterproto.uint32_field(8)


@dataclass(eq=False, repr=False)
class CUserChatRoomGroupState(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    time_joined: int = betterproto.uint32_field(2)
    user_chat_room_state: List["CUserChatRoomState"] = betterproto.message_field(3)
    desktop_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(4)
    mobile_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(5)
    time_last_group_ack: int = betterproto.uint32_field(6)
    unread_indicator_muted: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomCreateChatRoomGroupResponse(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    state: "CChatRoomGroupState" = betterproto.message_field(2)
    user_chat_state: "CUserChatRoomGroupState" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSaveChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSaveChatRoomGroupResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomRenameChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomRenameChatRoomGroupResponse(betterproto.Message):
    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupTaglineRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    tagline: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupTaglineResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupAvatarRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    avatar_sha: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupAvatarResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupWatchingBroadcastRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    watching_broadcast_accountid: int = betterproto.uint32_field(2)
    watching_broadcast_channel_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSetChatRoomGroupWatchingBroadcastResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomJoinMiniGameForChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomJoinMiniGameForChatRoomGroupResponse(betterproto.Message):
    minigame_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomEndMiniGameForChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    minigame_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomEndMiniGameForChatRoomGroupResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomMuteUserRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)
    expiration: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomMuteUserResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomKickUserRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)
    expiration: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomKickUserResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomSetUserBanStateRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)
    ban_state: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSetUserBanStateResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomRevokeInviteRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomRevokeInviteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomCreateRoleRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomCreateRoleResponse(betterproto.Message):
    actions: "CChatRoleActions" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomGetRolesRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetRolesResponse(betterproto.Message):
    roles: List["CChatRole"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomRenameRoleRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(2)
    name: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomRenameRoleResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomReorderRoleRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(2)
    ordinal: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomReorderRoleResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomDeleteRoleRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteRoleResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomGetRoleActionsRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomGetRoleActionsResponse(betterproto.Message):
    actions: List["CChatRoleActions"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomReplaceRoleActionsRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(2)
    actions: "CChatRoleActions" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomReplaceRoleActionsResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomAddRoleToUserRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(3)
    steamid: int = betterproto.fixed64_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomAddRoleToUserResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomGetRolesForUserRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomGetRolesForUserResponse(betterproto.Message):
    role_ids: List[int] = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteRoleFromUserRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    role_id: int = betterproto.uint64_field(3)
    steamid: int = betterproto.fixed64_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteRoleFromUserResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomJoinChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    invite_code: str = betterproto.string_field(2)
    chat_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomJoinChatRoomGroupResponse(betterproto.Message):
    state: "CChatRoomGroupState" = betterproto.message_field(1)
    user_chat_state: "CUserChatRoomGroupState" = betterproto.message_field(3)
    join_chat_id: int = betterproto.uint64_field(4)
    time_expire: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class CChatRoomInviteFriendToChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    steamid: int = betterproto.fixed64_field(2)
    chat_id: int = betterproto.uint64_field(3)
    skip_friendsui_check: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomInviteFriendToChatRoomGroupResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomLeaveChatRoomGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomLeaveChatRoomGroupResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomCreateChatRoomRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    name: str = betterproto.string_field(2)
    allow_voice: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomCreateChatRoomResponse(betterproto.Message):
    chat_room: "CChatRoomState" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteChatRoomRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteChatRoomResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomRenameChatRoomRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    name: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomRenameChatRoomResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomReorderChatRoomRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    move_after_chat_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomReorderChatRoomResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomSendChatMessageRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    message: str = betterproto.string_field(3)
    echo_to_sender: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomSendChatMessageResponse(betterproto.Message):
    modified_message: str = betterproto.string_field(1)
    server_timestamp: int = betterproto.uint32_field(2)
    ordinal: int = betterproto.uint32_field(3)
    message_without_bb_code: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomJoinVoiceChatRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomJoinVoiceChatResponse(betterproto.Message):
    voice_chatid: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomLeaveVoiceChatRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomLeaveVoiceChatResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageHistoryRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    last_time: int = betterproto.uint32_field(3)
    last_ordinal: int = betterproto.uint32_field(4)
    start_time: int = betterproto.uint32_field(5)
    start_ordinal: int = betterproto.uint32_field(6)
    max_count: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class ServerMessage(betterproto.Message):
    message: "EChatRoomServerMessage" = betterproto.enum_field(1)
    string_param: str = betterproto.string_field(2)
    accountid_param: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageHistoryResponse(betterproto.Message):
    messages: List["CChatRoomGetMessageHistoryResponseChatMessage"] = betterproto.message_field(1)
    more_available: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageHistoryResponseChatMessage(betterproto.Message):
    sender: int = betterproto.uint32_field(1)
    server_timestamp: int = betterproto.uint32_field(2)
    message: str = betterproto.string_field(3)
    ordinal: int = betterproto.uint32_field(4)
    server_message: "ServerMessage" = betterproto.message_field(5)
    deleted: bool = betterproto.bool_field(6)
    reactions: List["CChatRoomGetMessageHistoryResponseChatMessageMessageReaction"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageHistoryResponseChatMessageMessageReaction(betterproto.Message):
    reaction_type: "EChatRoomMessageReactionType" = betterproto.enum_field(1)
    reaction: str = betterproto.string_field(2)
    num_reactors: int = betterproto.uint32_field(3)
    has_user_reacted: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGetMyChatRoomGroupsRequest(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomGetChatRoomGroupSummaryResponse(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_group_name: str = betterproto.string_field(2)
    active_member_count: int = betterproto.uint32_field(3)
    active_voice_member_count: int = betterproto.uint32_field(4)
    default_chat_id: int = betterproto.uint64_field(5)
    chat_rooms: List["CChatRoomState"] = betterproto.message_field(6)
    clanid: int = betterproto.uint32_field(7)
    chat_group_tagline: str = betterproto.string_field(8)
    accountid_owner: int = betterproto.uint32_field(9)
    top_members: List[int] = betterproto.uint32_field(10)
    chat_group_avatar_sha: bytes = betterproto.bytes_field(11)
    rank: "EChatRoomGroupRank" = betterproto.enum_field(12)
    default_role_id: int = betterproto.uint64_field(13)
    role_ids: List[int] = betterproto.uint64_field(14)
    role_actions: List["CChatRoleActions"] = betterproto.message_field(15)
    watching_broadcast_accountid: int = betterproto.uint32_field(16)
    appid: int = betterproto.uint32_field(17)
    party_beacons: List["CChatPartyBeacon"] = betterproto.message_field(18)
    watching_broadcast_channel_id: int = betterproto.uint64_field(19)
    active_minigame_id: int = betterproto.uint64_field(20)
    avatar_ugc_url: str = betterproto.string_field(21)


@dataclass(eq=False, repr=False)
class CChatRoomSummaryPair(betterproto.Message):
    user_chat_group_state: "CUserChatRoomGroupState" = betterproto.message_field(1)
    group_summary: "CChatRoomGetChatRoomGroupSummaryResponse" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomGetMyChatRoomGroupsResponse(betterproto.Message):
    chat_room_groups: List["CChatRoomSummaryPair"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetChatRoomGroupStateRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetChatRoomGroupStateResponse(betterproto.Message):
    state: "CChatRoomGroupState" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetChatRoomGroupSummaryRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomSetAppChatRoomGroupForceActiveRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    requesting_app_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSetAppChatRoomGroupForceActiveResponse(betterproto.Message):
    result: int = betterproto.uint32_field(1)
    accounts_in_channel: List[int] = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSetAppChatRoomGroupStopForceActiveNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    requesting_app_id: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomAckChatMessageNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    timestamp: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomCreateInviteLinkRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    seconds_valid: int = betterproto.uint32_field(2)
    chat_id: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomCreateInviteLinkResponse(betterproto.Message):
    invite_code: str = betterproto.string_field(1)
    seconds_valid: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteLinkInfoRequest(betterproto.Message):
    invite_code: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteLinkInfoResponse(betterproto.Message):
    steamid_sender: int = betterproto.fixed64_field(3)
    time_expires: int = betterproto.uint32_field(4)
    chat_id: int = betterproto.uint64_field(6)
    group_summary: "CChatRoomGetChatRoomGroupSummaryResponse" = betterproto.message_field(8)
    user_chat_group_state: "CUserChatRoomGroupState" = betterproto.message_field(9)
    time_kick_expire: int = betterproto.uint32_field(10)
    banned: bool = betterproto.bool_field(11)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteInfoRequest(betterproto.Message):
    steamid_invitee: int = betterproto.fixed64_field(1)
    chat_group_id: int = betterproto.uint64_field(2)
    chat_id: int = betterproto.uint64_field(3)
    invite_code: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteInfoResponse(betterproto.Message):
    group_summary: "CChatRoomGetChatRoomGroupSummaryResponse" = betterproto.message_field(1)
    time_kick_expire: int = betterproto.uint32_field(2)
    banned: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteLinksForGroupRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteLinksForGroupResponse(betterproto.Message):
    invite_links: List["CChatRoomGetInviteLinksForGroupResponseLinkInfo"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteLinksForGroupResponseLinkInfo(betterproto.Message):
    invite_code: str = betterproto.string_field(1)
    steamid_creator: int = betterproto.fixed64_field(2)
    time_expires: int = betterproto.uint32_field(3)
    chat_id: int = betterproto.uint64_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGetBanListRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetBanListResponse(betterproto.Message):
    bans: List["CChatRoomGetBanListResponseBanInfo"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetBanListResponseBanInfo(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    accountid_actor: int = betterproto.uint32_field(2)
    time_banned: int = betterproto.uint32_field(3)
    ban_reason: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteListRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGroupInvite(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    accountid_actor: int = betterproto.uint32_field(2)
    time_invited: int = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomGetInviteListResponse(betterproto.Message):
    invites: List["CChatRoomGroupInvite"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteInviteLinkRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    invite_code: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteInviteLinkResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomSetSessionActiveChatRoomGroupsRequest(betterproto.Message):
    chat_group_ids: List[int] = betterproto.uint64_field(1)
    chat_groups_data_requested: List[int] = betterproto.uint64_field(2)
    virtualize_members_threshold: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSetSessionActiveChatRoomGroupsResponse(betterproto.Message):
    chat_states: List["CChatRoomGroupState"] = betterproto.message_field(1)
    virtualize_members_chat_group_ids: List[int] = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSetUserChatGroupPreferencesRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_group_preferences: "CChatRoomSetUserChatGroupPreferencesRequestChatGroupPreferences" = (
        betterproto.message_field(2)
    )
    chat_room_preferences: List[
        "CChatRoomSetUserChatGroupPreferencesRequestChatRoomPreferences"
    ] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSetUserChatGroupPreferencesRequestChatGroupPreferences(betterproto.Message):
    desktop_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(1)
    mobile_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(2)
    unread_indicator_muted: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomSetUserChatGroupPreferencesRequestChatRoomPreferences(betterproto.Message):
    chat_id: int = betterproto.uint64_field(1)
    desktop_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(2)
    mobile_notification_level: "EChatRoomNotificationLevel" = betterproto.enum_field(3)
    unread_indicator_muted: bool = betterproto.bool_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomSetUserChatGroupPreferencesResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomDeleteChatMessagesRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    messages: List["CChatRoomDeleteChatMessagesRequestMessage"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteChatMessagesRequestMessage(betterproto.Message):
    server_timestamp: int = betterproto.uint32_field(1)
    ordinal: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomDeleteChatMessagesResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CChatRoomUpdateMemberListViewNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    view_id: int = betterproto.uint64_field(2)
    start: int = betterproto.int32_field(3)
    end: int = betterproto.int32_field(4)
    client_changenumber: int = betterproto.int32_field(5)
    delete_view: bool = betterproto.bool_field(6)
    persona_subscribe_accountids: List[int] = betterproto.int32_field(7)
    persona_unsubscribe_accountids: List[int] = betterproto.int32_field(8)


@dataclass(eq=False, repr=False)
class CChatRoomSearchMembersRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    search_id: int = betterproto.uint64_field(2)
    search_text: str = betterproto.string_field(3)
    max_results: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class CChatRoomSearchMembersResponse(betterproto.Message):
    matching_members: List["CChatRoomSearchMembersResponseMemberMatch"] = betterproto.message_field(1)
    status_flags: int = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomSearchMembersResponseMemberMatch(betterproto.Message):
    accountid: int = betterproto.int32_field(1)
    persona: "CMsgClientPersonaStateFriend" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class CChatRoomUpdateMessageReactionRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    server_timestamp: int = betterproto.uint32_field(3)
    ordinal: int = betterproto.uint32_field(4)
    reaction_type: "EChatRoomMessageReactionType" = betterproto.enum_field(5)
    reaction: str = betterproto.string_field(6)
    is_add: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomUpdateMessageReactionResponse(betterproto.Message):
    num_reactors: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageReactionReactorsRequest(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    server_timestamp: int = betterproto.uint32_field(3)
    ordinal: int = betterproto.uint32_field(4)
    reaction_type: "EChatRoomMessageReactionType" = betterproto.enum_field(5)
    reaction: str = betterproto.string_field(6)
    limit: int = betterproto.uint32_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomGetMessageReactionReactorsResponse(betterproto.Message):
    reactors: List[int] = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class CClanChatRoomsGetClanChatRoomInfoRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    autocreate: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CClanChatRoomsGetClanChatRoomInfoResponse(betterproto.Message):
    chat_group_summary: "CChatRoomGetChatRoomGroupSummaryResponse" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CClanChatRoomsSetClanChatRoomPrivateRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    chat_room_private: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class CClanChatRoomsSetClanChatRoomPrivateResponse(betterproto.Message):
    chat_room_private: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class CChatMentions(betterproto.Message):
    mention_all: bool = betterproto.bool_field(1)
    mention_here: bool = betterproto.bool_field(2)
    ids: List[int] = betterproto.uint32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomIncomingChatMessageNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    steamid_sender: int = betterproto.fixed64_field(3)
    message: str = betterproto.string_field(4)
    timestamp: int = betterproto.uint32_field(5)
    mentions: "CChatMentions" = betterproto.message_field(6)
    ordinal: int = betterproto.uint32_field(7)
    server_message: "ServerMessage" = betterproto.message_field(8)
    message_no_bbcode: str = betterproto.string_field(9)
    chat_name: str = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class CChatRoomChatMessageModifiedNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    messages: List["CChatRoomChatMessageModifiedNotificationChatMessage"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomChatMessageModifiedNotificationChatMessage(betterproto.Message):
    server_timestamp: int = betterproto.uint32_field(1)
    ordinal: int = betterproto.uint32_field(2)
    deleted: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomMemberStateChangeNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    member: "CChatRoomMember" = betterproto.message_field(2)
    change: "EChatRoomMemberStateChange" = betterproto.enum_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomChatRoomHeaderStateNotification(betterproto.Message):
    header_state: "CChatRoomGroupHeaderState" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomChatRoomGroupRoomsChangeNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    default_chat_id: int = betterproto.uint64_field(2)
    chat_rooms: List["CChatRoomState"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomNotifyShouldRejoinChatRoomVoiceChatNotification(betterproto.Message):
    chat_id: int = betterproto.uint64_field(1)
    chat_group_id: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class ChatRoomClientNotifyChatGroupUserStateChangedNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    user_chat_group_state: "CUserChatRoomGroupState" = betterproto.message_field(2)
    group_summary: "CChatRoomGetChatRoomGroupSummaryResponse" = betterproto.message_field(3)
    user_action: "EChatRoomMemberStateChange" = betterproto.enum_field(4)


@dataclass(eq=False, repr=False)
class ChatRoomClientNotifyChatRoomDisconnectNotification(betterproto.Message):
    chat_group_ids: List[int] = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class CChatRoomMemberListView(betterproto.Message):
    start: int = betterproto.int32_field(3)
    end: int = betterproto.int32_field(4)
    total_count: int = betterproto.int32_field(5)
    client_changenumber: int = betterproto.int32_field(6)
    server_changenumber: int = betterproto.int32_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomMemberSummaryCounts(betterproto.Message):
    ingame: int = betterproto.int32_field(1)
    online: int = betterproto.int32_field(2)
    offline: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomClientMemberListViewUpdatedNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    view_id: int = betterproto.uint64_field(2)
    view: "CChatRoomMemberListView" = betterproto.message_field(3)
    members: List["CChatRoomClientMemberListViewUpdatedNotificationMemberListViewEntry"] = betterproto.message_field(4)
    status_flags: int = betterproto.uint32_field(5)
    member_summary: "CChatRoomMemberSummaryCounts" = betterproto.message_field(6)
    subscribed_personas: List["CMsgClientPersonaStateFriend"] = betterproto.message_field(7)


@dataclass(eq=False, repr=False)
class CChatRoomClientMemberListViewUpdatedNotificationMemberListViewEntry(betterproto.Message):
    rank: int = betterproto.int32_field(1)
    accountid: int = betterproto.uint32_field(2)
    persona: "CMsgClientPersonaStateFriend" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CChatRoomMessageReactionNotification(betterproto.Message):
    chat_group_id: int = betterproto.uint64_field(1)
    chat_id: int = betterproto.uint64_field(2)
    server_timestamp: int = betterproto.uint32_field(3)
    ordinal: int = betterproto.uint32_field(4)
    reactor: int = betterproto.fixed64_field(5)
    reaction_type: "EChatRoomMessageReactionType" = betterproto.enum_field(6)
    reaction: str = betterproto.string_field(7)
    is_add: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotification(betterproto.Message):
    metrics_run_id: int = betterproto.uint32_field(1)
    client_build: int = betterproto.uint32_field(2)
    metrics_version: int = betterproto.uint32_field(3)
    in_web: bool = betterproto.bool_field(4)
    settings: "CChatUsabilityClientUsabilityMetricsNotificationSettings" = betterproto.message_field(10)
    voice_settings: "CChatUsabilityClientUsabilityMetricsNotificationVoiceSettings" = betterproto.message_field(11)
    ui_state: "CChatUsabilityClientUsabilityMetricsNotificationUiState" = betterproto.message_field(12)
    metrics: "CChatUsabilityClientUsabilityMetricsNotificationMetrics" = betterproto.message_field(13)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotificationSettings(betterproto.Message):
    notifications_show_ingame: bool = betterproto.bool_field(1)
    notifications_show_online: bool = betterproto.bool_field(2)
    notifications_show_message: bool = betterproto.bool_field(3)
    notifications_events_and_announcements: bool = betterproto.bool_field(4)
    sounds_play_ingame: bool = betterproto.bool_field(5)
    sounds_play_online: bool = betterproto.bool_field(6)
    sounds_play_message: bool = betterproto.bool_field(7)
    sounds_events_and_announcements: bool = betterproto.bool_field(8)
    always_new_chat_window: bool = betterproto.bool_field(9)
    force_alphabetic_friend_sorting: bool = betterproto.bool_field(10)
    chat_flash_mode: int = betterproto.int32_field(11)
    remember_open_chats: bool = betterproto.bool_field(12)
    compact_quick_access: bool = betterproto.bool_field(13)
    compact_friends_list: bool = betterproto.bool_field(14)
    notifications_show_chat_room_notification: bool = betterproto.bool_field(15)
    sounds_play_chat_room_notification: bool = betterproto.bool_field(16)
    hide_offline_friends_in_tag_groups: bool = betterproto.bool_field(17)
    hide_categorized_friends: bool = betterproto.bool_field(18)
    categorize_in_game_friends_by_game: bool = betterproto.bool_field(19)
    chat_font_size: int = betterproto.int32_field(20)
    use24_hour_clock: bool = betterproto.bool_field(21)
    do_not_disturb_mode: bool = betterproto.bool_field(22)
    disable_embed_inlining: bool = betterproto.bool_field(23)
    sign_into_friends: bool = betterproto.bool_field(24)
    animated_avatars: bool = betterproto.bool_field(25)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotificationVoiceSettings(betterproto.Message):
    voice_input_gain: float = betterproto.float_field(1)
    voice_output_gain: float = betterproto.float_field(2)
    noise_gate_level: int = betterproto.int32_field(3)
    voice_use_echo_cancellation: bool = betterproto.bool_field(4)
    voice_use_noise_cancellation: bool = betterproto.bool_field(5)
    voice_use_auto_gain_control: bool = betterproto.bool_field(6)
    selected_non_default_mic: bool = betterproto.bool_field(7)
    selected_non_default_output: bool = betterproto.bool_field(8)
    push_to_talk_enabled: bool = betterproto.bool_field(9)
    push_to_mute_enabled: bool = betterproto.bool_field(10)
    play_ptt_sounds: bool = betterproto.bool_field(11)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotificationUiState(betterproto.Message):
    friends_list_height: int = betterproto.int32_field(1)
    friends_list_width: int = betterproto.int32_field(2)
    friends_list_docked: bool = betterproto.bool_field(3)
    friends_list_collapsed: bool = betterproto.bool_field(4)
    friends_list_group_chats_height: int = betterproto.int32_field(5)
    friends_list_visible: bool = betterproto.bool_field(6)
    chat_popups_opened: int = betterproto.int32_field(7)
    group_chat_tabs_opened: int = betterproto.int32_field(8)
    friend_chat_tabs_opened: int = betterproto.int32_field(9)
    chat_window_width: int = betterproto.int32_field(10)
    chat_window_height: int = betterproto.int32_field(11)
    category_collapse: "CChatUsabilityClientUsabilityMetricsNotificationUiStateCategoryCollapseState" = (
        betterproto.message_field(12)
    )
    group_chat_left_col_collapsed: int = betterproto.int32_field(13)
    group_chat_right_col_collapsed: int = betterproto.int32_field(14)
    in_one_on_one_voice_chat: bool = betterproto.bool_field(15)
    in_group_voice_chat: bool = betterproto.bool_field(16)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotificationUiStateCategoryCollapseState(betterproto.Message):
    in_game_collapsed: bool = betterproto.bool_field(1)
    online_collapsed: bool = betterproto.bool_field(2)
    offline_collapsed: bool = betterproto.bool_field(3)
    game_groups_collapsed: int = betterproto.int32_field(4)
    categories_collapsed: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class CChatUsabilityClientUsabilityMetricsNotificationMetrics(betterproto.Message):
    friends_count: int = betterproto.int32_field(1)
    friends_category_count: int = betterproto.int32_field(2)
    friends_categorized_count: int = betterproto.int32_field(3)
    friends_online_count: int = betterproto.int32_field(4)
    friends_in_game_count: int = betterproto.int32_field(5)
    friends_in_game_singleton_count: int = betterproto.int32_field(6)
    game_group_count: int = betterproto.int32_field(7)
    friends_favorite_count: int = betterproto.int32_field(8)
    group_chat_count: int = betterproto.int32_field(9)
    group_chat_favorite_count: int = betterproto.int32_field(10)


@dataclass(eq=False, repr=False)
class CChatUsabilityRequestClientUsabilityMetricsNotification(betterproto.Message):
    metrics_run_id: int = betterproto.uint32_field(1)



"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import re
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Any, Optional, TypeVar, overload

from typing_extensions import Literal, TypedDict

from .enums import IntEnum, ReviewType
from .models import URL

if TYPE_CHECKING:
    from .utils import Intable

__all__ = (
    "TF2",
    "Game",
    "CSGO",
    "DOTA2",
    "LFD2",
    "STEAM",
    "CUSTOM_GAME",
    "UserGame",
    "WishlistGame",
    "FetchedGame",
)

T = TypeVar("T")
APP_ID_MAX = 2 ** 32


class Games(IntEnum):
    Team_Fortress_2 = 440
    Left_4_Dead_2 = 550
    DOTA_2 = 570
    Counter_Strike_Global__Offensive = 730
    Steam = 753


class GameDict(TypedDict):
    name: str
    appid: str
    playtime_forever: int
    img_icon_url: str
    img_logo_url: str
    has_community_visible_stats: bool


class GameToDict(TypedDict, total=False):
    game_id: str
    game_extra_info: str


class WishlistGameDict(TypedDict):
    name: str
    capsule: str
    review_score: int
    review_desc: str
    reviews_total: str
    reviews_percent: int
    release_date: int
    release_string: str
    platform_icons: str
    subs: list[dict]
    type: str
    screenshots: list[str]
    review_css: str
    priority: int
    added: int
    background: str
    rank: int
    tags: list[str]
    is_free_game: bool
    win: bool
    mac: bool
    linux: bool


class PackageGroupsDict(TypedDict):
    name: Literal["default", "subscriptions"]
    title: str
    description: str
    selection_text: str
    display_type: Literal[0, 1]
    is_recurring_subscription: bool
    subs: list[dict[str, str]]


class FetchedGameDict(TypedDict):
    # https://wiki.teamfortress.com/wiki/User:RJackson/StorefrontAPI#Result_data_3
    type: Literal["game", "dlc", "demo", "advertising", "mod", "video"]
    name: str
    steam_appid: int
    required_age: int
    is_free: bool
    controller_support: Literal["partial", "full"]
    dlc: list[int]
    detailed_description: str
    short_description: str
    fullgame: dict
    supported_languages: str
    header_image: str
    pc_requirements: list[dict[str, str]]
    mac_requirements: list[dict[str, str]]
    linux_requirements: list[dict[str, str]]
    legal_notice: str
    developers: list[str]
    publishers: list[str]
    demos: list[dict]
    price_overview: list[dict]
    package_groups: list[PackageGroupsDict]
    platforms: dict[str, bool]
    metacritic: list[dict[str, str]]
    categories: list[dict[str, str]]
    release_date: dict[str, str]
    background: str
    website: str
    movies: list[dict[str, Any]]


class Game:
    """Represents a Steam game.

    Parameters
    ----------
    title: Optional[:class:`str`]
        The game's title.
    id: Optional[:class:`int`]
        The game's app ID.
    context_id: Optional[:class:`int`]
        The game's context ID by default 2.

    Attributes
    -----------
    title: Optional[:class:`str`]
        The game's title.
    id: :class:`int`
        The game's app ID.
    context_id: :class:`int`
        The context id of the game normally ``2``.
    """

    __slots__ = (
        "id",
        "title",
        "context_id",
    )

    @overload
    def __init__(self, *, id: Intable, context_id: Optional[int] = None):
        ...

    @overload
    def __init__(self, *, title: str, context_id: Optional[int] = None):
        ...

    @overload
    def __init__(self, *, id: Intable, title: str, context_id: Optional[int] = None):
        ...

    def __init__(self, *, id: Optional[Intable] = None, title: Optional[str] = None, context_id: Optional[int] = None):
        if title is None and id is None:
            raise TypeError("__init__() missing a required keyword argument: 'id' or 'title'")

        if title is None:
            try:
                id = int(id)
            except (ValueError, TypeError):
                raise ValueError(f"id expected to support int()")
            try:
                title = Games(id).name.replace("__", "-").replace("_", " ")
            except ValueError:
                title = None
            else:
                if title == "Steam" and context_id is None:
                    context_id = 6
        elif id is None:
            try:
                id = Games[title.replace(" ", "_").replace("-", "__")].value
            except KeyError:
                id = 0
        else:
            try:
                id = int(id)
            except (ValueError, TypeError):
                raise ValueError("id must be an int") from None

        if id < 0:
            raise ValueError("id cannot be negative")

        self.id: Optional[int] = id
        self.title: Optional[str] = title
        self.context_id: int = 2 if context_id is None else context_id

    def __str__(self) -> str:
        return self.title or ""

    def __repr__(self) -> str:
        attrs = ("title", "id", "context_id")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"{self.__class__.__name__}({', '.join(resolved)})"

    def __eq__(self, other: Any) -> bool:
        if not isinstance(other, Game):
            return NotImplemented
        return self.id == other.id or self.title == other.title

    def to_dict(self) -> GameToDict:
        """dict[:class:`str`, :class:`str`]: The dict representation of the game used to set presences."""
        return (
            {"game_id": str(self.id)}
            if self.is_steam_game()
            else {"game_id": str(self.id), "game_extra_info": self.title}
        )

    def is_steam_game(self) -> bool:
        """:class:`bool`: Whether the game could be a Steam game."""
        return self.id <= APP_ID_MAX

    @property
    def url(self) -> str:
        """:class:`str`: The game's url on store.steampowered if applicable."""
        if not self.id:
            raise ValueError("Games without id's can't have associated URLs")
        return f"{URL.STORE}/apps/{self.id}"


TF2 = Game(title="Team Fortress 2")
DOTA2 = Game(title="DOTA 2")
CSGO = Game(title="Counter Strike Global-Offensive")
LFD2 = Game(title="Left 4 Dead 2")
STEAM = Game(title="Steam", context_id=6)


def CUSTOM_GAME(title: str) -> Game:
    """Create a custom game instance for :meth:`~steam.Client.change_presence`.
    The :attr:`Game.id` will be set to ``15190414816125648896`` and the :attr:`Game.context_id` to ``None``.

    Example: ::

        await client.change_presence(game=steam.CUSTOM_GAME('my cool game'))

    Parameters
    ----------
    title: :class:`str`
        The name of the game to set your playing status to.

    Returns
    -------
    class:`Game`
        The created custom game.
    """
    return Game(title=title, id=15190414816125648896, context_id=None)


class UserGame(Game):
    """Represents a Steam game fetched by :meth:`steam.User.games`

    Attributes
    ----------
    total_play_time: :class:`datetime.timedelta`
        The total time the game has been played for.
    icon_url: Optional[:class:`str`]
        The icon url of the game.
    logo_url: Optional[:class:`str`]
        The logo url of the game.
    """

    __slots__ = (
        "total_play_time",
        "icon_url",
        "logo_url",
        "_stats_visible",
    )

    def __init__(self, data: GameDict):
        super().__init__(id=data.get("appid"), title=data.get("name"))
        self.total_play_time: timedelta = timedelta(minutes=data.get("playtime_forever", 0))
        self.icon_url: str = (
            f"https://cdn.cloudflare.steamstatic.com/steamcommunity/public/images/apps/{self.id}/"
            f"{data['img_icon_url']}.jpg"
        )
        self.logo_url: str = (
            f"https://cdn.cloudflare.steamstatic.com/steamcommunity/public/images/apps/{self.id}/"
            f"{data['img_logo_url']}.jpg"
        )
        self._stats_visible = data.get("has_community_visible_stats", False)

    def has_visible_stats(self) -> bool:
        """:class:`bool`: Whether the game has publicly visible stats.
        Only applies to a :class:`~steam.User`'s games from :meth:`~steam.User.games`.
        """
        return self._stats_visible


class WishlistGame(Game):
    """Represents a Steam game fetched by :meth:`steam.User.wishlist`

    Attributes
    ----------
    priority: :class:`int`
        The priority of the game in the wishlist.
    added_at: :class:`.datetime`
        The time the the game was added to their wishlist.
    created_at: :class:`.datetime`
        The time the game was uploaded at.
    background_url: :class:`str`
        The background URL of the game.
    rank: :class:`int`
        The global rank of the game by popularity.
    review_status: :class:`.ReviewType`
        The review status of the game.
    score: :class:`int`
        The score of the game out of ten.
    screenshots: list[:class:`str`]
        The screenshots of the game.
    tags: list[:class:`str`]
        The tags of the game.
    total_reviews: :class:`int`
        The total number reviews for the game.
    type: :class:`str`
        The type of the app.
    logo_url: :class:`str`
        The logo url of the game.
    """

    __slots__ = (
        "priority",
        "added_at",
        "background_url",
        "created_at",
        "logo_url",
        "rank",
        "review_status",
        "score",
        "screenshots",
        "tags",
        "total_reviews",
        "type",
        "_free",
        "_on_linux",
        "_on_mac_os",
        "_on_windows",
    )

    def __init__(self, id: int, data: WishlistGameDict):
        super().__init__(id=id, title=data["name"])
        self.logo_url = data["capsule"]
        self.score = data["review_score"]
        self.total_reviews = int(data["reviews_total"].replace(",", ""))
        self.review_status = ReviewType[data["review_desc"].replace(" ", "")]
        self.created_at = datetime.utcfromtimestamp(int(data["release_date"]))
        self.type = data["type"]
        self.screenshots: list[str] = [
            f"https://cdn.cloudflare.steamstatic.com/steam/apps/{self.id}/{screenshot_url}"
            for screenshot_url in data["screenshots"]
        ]
        self.added_at = datetime.utcfromtimestamp(data["added"])
        self.background_url = data["background"]
        self.tags = data["tags"]
        self.rank = data["rank"]
        self.priority = int(data["priority"])

        self._free = data["is_free_game"]
        self._on_windows = bool(data.get("win", False))
        self._on_mac_os = bool(data.get("mac", False))
        self._on_linux = bool(data.get("linux", False))

    def is_free(self) -> bool:
        """:class:`bool`: Whether or not the game is free to download."""
        return self._free

    def is_on_windows(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on Windows."""
        return self._on_windows

    def is_on_mac_os(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on MacOS."""
        return self._on_mac_os

    def is_on_linux(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on Linux."""
        return self._on_linux


class Movie:
    __slots__ = ("name", "id", "url", "created_at")

    def __init__(self, movie: dict[str, Any]):
        self.name: str = movie["name"]
        self.id: int = movie["id"]
        self.url: str = movie["mp4"]["max"]
        match = re.search(r"t=(\d+)", self.url)
        self.created_at = datetime.utcfromtimestamp(int(match.group(1))) if match else None

    def __repr__(self) -> str:
        attrs = ("name", "id", "url", "created_at")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"<Movie {' '.join(resolved)}>"


class FetchedGame(Game):
    """Represents a Steam game fetched by :meth:`steam.Client.fetch_game`

    Attributes
    ----------
    created_at: :class:`.datetime`
        The time the game was uploaded at.
    background_url: :class:`str`
        The background URL of the game.
    type: :class:`str`
        The type of the app.
    logo_url: :class:`str`
        The logo URL of the game.
    dlc: list[:class:`Game`]
        The game's downloadable content.
    website_url: :class:`str`
        The website URL of the game.
    developers: list[:class:`str`]
        The developers of the game.
    publishers: list[:class:`str`]
        The publishers of the game.
    description: :class:`str`
        The short description of the game.
    full_description: :class:`str`
        The full description of the game.
    movies: list[:class:`Movie`]
        A list of the game's movies, each of which has ``name``, ``id``, ``url`` and optional ``created_at`` attributes.
    """

    __slots__ = (
        "logo_url",
        "background_url",
        "created_at",
        "type",
        "dlc",
        "website_url",
        "developers",
        "publishers",
        "description",
        "full_description",
        "movies",
        "_free",
        "_on_windows",
        "_on_mac_os",
        "_on_linux",
    )

    def __init__(self, data: FetchedGameDict):
        super().__init__(id=data["steam_appid"], title=data["name"])
        self.logo_url = data["header_image"]
        self.background_url = data["background"]
        self.created_at = (
            datetime.strptime(data["release_date"]["date"], "%d %b, %Y") if data["release_date"]["date"] else None
        )
        self.type = data["type"]

        self.dlc = [Game(id=dlc_id) for dlc_id in data["dlc"]] if data.get("dlc") else None
        self.website_url = data.get("website")
        self.developers = data["developers"]
        self.publishers = data["publishers"]
        self.description = data["short_description"]
        self.full_description = data["detailed_description"]

        self.movies = [Movie(movie) for movie in data["movies"]] if data.get("movies") else None

        self._free = data["is_free"]
        self._on_windows = bool(data["platforms"].get("windows", False))
        self._on_mac_os = bool(data["platforms"].get("mac", False))
        self._on_linux = bool(data["platforms"].get("linux", False))

    def is_free(self) -> bool:
        """:class:`bool`: Whether or not the game is free to download."""
        return self._free

    def is_on_windows(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on Windows."""
        return self._on_windows

    def is_on_mac_os(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on MacOS."""
        return self._on_mac_os

    def is_on_linux(self) -> bool:
        """:class:`bool`: Whether or not the game is able to be played on Linux."""
        return self._on_linux



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_twofactor.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class CTwoFactorStatusRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CTwoFactorStatusResponse(betterproto.Message):
    state: int = betterproto.uint32_field(1)
    inactivation_reason: int = betterproto.uint32_field(2)
    authenticator_type: int = betterproto.uint32_field(3)
    authenticator_allowed: bool = betterproto.bool_field(4)
    steamguard_scheme: int = betterproto.uint32_field(5)
    token_gid: str = betterproto.string_field(6)
    email_validated: bool = betterproto.bool_field(7)
    device_identifier: str = betterproto.string_field(8)
    time_created: int = betterproto.uint32_field(9)
    revocation_attempts_remaining: int = betterproto.uint32_field(10)
    classified_agent: str = betterproto.string_field(11)
    allow_external_authenticator: bool = betterproto.bool_field(12)
    time_transferred: int = betterproto.uint32_field(13)


@dataclass(eq=False, repr=False)
class CTwoFactorAddAuthenticatorRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    authenticator_time: int = betterproto.uint64_field(2)
    serial_number: int = betterproto.fixed64_field(3)
    authenticator_type: int = betterproto.uint32_field(4)
    device_identifier: str = betterproto.string_field(5)
    sms_phone_id: str = betterproto.string_field(6)
    http_headers: List[str] = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class CTwoFactorAddAuthenticatorResponse(betterproto.Message):
    shared_secret: bytes = betterproto.bytes_field(1)
    serial_number: int = betterproto.fixed64_field(2)
    revocation_code: str = betterproto.string_field(3)
    uri: str = betterproto.string_field(4)
    server_time: int = betterproto.uint64_field(5)
    account_name: str = betterproto.string_field(6)
    token_gid: str = betterproto.string_field(7)
    identity_secret: bytes = betterproto.bytes_field(8)
    secret_1: bytes = betterproto.bytes_field(9)
    status: int = betterproto.int32_field(10)


@dataclass(eq=False, repr=False)
class CTwoFactorSendEmailRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    email_type: int = betterproto.uint32_field(2)
    include_activation_code: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class CTwoFactorSendEmailResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CTwoFactorFinalizeAddAuthenticatorRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)
    authenticator_code: str = betterproto.string_field(2)
    authenticator_time: int = betterproto.uint64_field(3)
    activation_code: str = betterproto.string_field(4)
    http_headers: List[str] = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class CTwoFactorFinalizeAddAuthenticatorResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    want_more: bool = betterproto.bool_field(2)
    server_time: int = betterproto.uint64_field(3)
    status: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class CTwoFactorRemoveAuthenticatorRequest(betterproto.Message):
    revocation_code: str = betterproto.string_field(2)
    revocation_reason: int = betterproto.uint32_field(5)
    steamguard_scheme: int = betterproto.uint32_field(6)
    remove_all_steamguard_cookies: bool = betterproto.bool_field(7)


@dataclass(eq=False, repr=False)
class CTwoFactorRemoveAuthenticatorResponse(betterproto.Message):
    success: bool = betterproto.bool_field(1)
    server_time: int = betterproto.uint64_field(3)
    revocation_attempts_remaining: int = betterproto.uint32_field(5)


@dataclass(eq=False, repr=False)
class CTwoFactorCreateEmergencyCodesRequest(betterproto.Message):
    code: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CTwoFactorCreateEmergencyCodesResponse(betterproto.Message):
    codes: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CTwoFactorDestroyEmergencyCodesRequest(betterproto.Message):
    steamid: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class CTwoFactorDestroyEmergencyCodesResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CTwoFactorValidateTokenRequest(betterproto.Message):
    code: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class CTwoFactorValidateTokenResponse(betterproto.Message):
    valid: bool = betterproto.bool_field(1)



"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Optional

from ...abc import Channel, Message, Messageable, _EndPointReturnType

if TYPE_CHECKING:
    from datetime import datetime

    from ...clan import Clan
    from ...group import Group
    from ...iterators import AsyncIterator
    from ...state import ConnectionState
    from ...user import User
    from .bot import Bot
    from .cog import Cog
    from .commands import Command
    from .utils import Shlex


__all__ = ("Context",)


class Context(Messageable):
    """Represents the context of a command.

    Attributes
    ----------
    message: :class:`~steam.Message`
        The message the context was generated from.
    prefix: :class:`str`
        The prefix of the message the context was generated from
    author: :class:`~steam.User`
        The author of the message.
    channel: :class:`~steam.Channel`
        The channel the message was sent in.
    clan: Optional[:class:`~steam.Clan`]
        The clan the message was sent in ``None`` if the message wasn't sent in a clan.
    group: Optional[:class:`~steam.Group`]
        The group the message was sent in ``None`` if the message wasn't sent in a group.
    bot: :class:`~steam.ext.commands.Bot`
        The bot instance.
    command: Optional[:class:`~steam.ext.commands.Command`]
        The command the context is attached to.
    cog: Optional[:class:`~steam.ext.commands.Cog`]
        The cog the command is in.
    """

    def __init__(self, **attrs: Any):
        self.bot: Bot = attrs["bot"]
        self.message: Message = attrs["message"]
        self.prefix: str = attrs["prefix"]

        self.command: Optional[Command] = attrs.get("command")
        self.cog: Optional[Cog] = self.command.cog if self.command is not None else None
        self.lex: Optional[Shlex] = attrs.get("lex")
        self.invoked_with: Optional[str] = attrs.get("invoked_with")

        self.author: User = self.message.author
        self.channel: Channel = self.message.channel
        self.clan: Clan = self.message.clan
        self.group: Group = self.message.group
        self._state: ConnectionState = self.message._state

        self.args: Optional[tuple[Any, ...]] = None
        self.kwargs: Optional[dict[str, Any]] = None
        self.command_failed: bool = False

    def _get_message_endpoint(self) -> _EndPointReturnType:
        return self.channel._get_message_endpoint()

    def _get_image_endpoint(self) -> _EndPointReturnType:
        return self.channel._get_image_endpoint()

    async def invoke(self) -> None:
        """|coro|
        A shortcut method that invokes the current context using :meth:`~steam.ext.commands.Command.invoke`.

        Equivalent to::

            await ctx.command.invoke(ctx)
        """
        await self.command.invoke(self)

    def history(
        self,
        limit: Optional[int] = 100,
        before: Optional[datetime] = None,
        after: Optional[datetime] = None,
    ) -> AsyncIterator[Message]:
        """|coro|
        A shortcut method that gets the current channel's history.

        Equivalent to::

            return ctx.channel.history(**kwargs)
        """
        return self.channel.history(limit=limit, before=before, after=after)

    @property
    def valid(self) -> bool:
        """:class:`bool`: Whether or not the context could be invoked."""
        return self.prefix is not None and self.command is not None



"""
The MIT License (MIT)

Copyright (c) 2015 Rossen Georgiev <rossen@rgp.io>
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

This contains a large amount of
https://github.com/ValvePython/steam/blob/master/steam/game_servers.py
"""

from __future__ import annotations

import asyncio
import socket
import struct
from binascii import crc32
from bz2 import decompress
from collections.abc import Callable
from contextlib import AbstractAsyncContextManager, asynccontextmanager
from datetime import timedelta
from typing import TYPE_CHECKING, Any, Generic, NamedTuple, Optional, TypeVar, Union

from typing_extensions import Literal

from .abc import SteamID
from .enums import Enum, GameServerRegion
from .game import Game
from .utils import StructIO

if TYPE_CHECKING:
    from .protobufs.steammessages_gameservers import CGameServersGetServerListResponseServer as GameServerProto

T = TypeVar("T")
Q = TypeVar("Q", bound="Query")

__all__ = (
    "Query",
    "GameServer",
    "ServerPlayer",
)


class Operator(Enum):
    # fmt: off
    div  = "\\"
    or_  = "\\nor\\"
    and_ = "\\nand\\"
    # fmt: on

    def format(self, query_1: str, query_2: str) -> str:
        return f"{query_1}{query_2}" if self is Operator.div else rf"{self.value}[{query_1}{query_2}]"


class QueryAll:
    def __repr__(self) -> str:
        return "all"

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, self.__class__)

    query = ""


class QueryMeta(type):
    @property
    def not_empty(cls) -> Query[Q]:
        """Fetches servers that are not empty."""
        return Query(r"\empty\1")

    @property
    def empty(cls) -> Query[Q]:
        """Fetches servers that are empty."""
        return Query(r"\noplayers\1")

    @property
    def proxy(cls) -> Query[Q]:
        """Fetches servers that are spectator proxies."""
        return Query(r"\proxy\1")

    @property
    def whitelisted(cls) -> Query[Q]:
        """Fetches servers that are whitelisted."""
        return Query(r"\white\1")

    @property
    def dedicated(cls) -> Query[Q]:
        """Fetches servers that are running dedicated."""
        return Query(r"\dedicated\1")

    @property
    def secure(cls) -> Query[Q]:
        """Fetches servers that are using anti-cheat technology (VAC, but potentially others as well)."""
        return Query(r"\secure\1")

    @property
    def linux(cls) -> Query[Q]:
        """Fetches servers running on a Linux platform."""
        return Query(r"\linux\1")

    @property
    def no_password(cls) -> Query[Q]:
        """Fetches servers that are not password protected."""
        return Query(r"\password\0")

    @property
    def not_full(cls) -> Query[Q]:
        """Fetches servers that are not full."""
        return Query(r"\full\1")

    @property
    def unique_addresses(cls) -> Query[Q]:
        """Fetches only one server for each unique IP address matched."""
        return Query(r"\collapse_addr_hash\1")

    @property
    def version_match(cls) -> Query[str]:
        """Fetches servers running version "x" (``"*"`` is wildcard)."""
        return Query("\\version_match\\", type=str)

    @property
    def name_match(cls) -> Query[str]:
        """Fetches servers with their hostname matching "x" (``"*"`` is wildcard)."""
        return Query("\\name_match\\", type=str)

    @property
    def running_mod(cls) -> Query[str]:
        """Fetches servers running the specified modification (e.g. cstrike)."""
        return Query("\\gamedir\\", type=str)

    @property
    def running_map(cls) -> Query[str]:
        """Fetches servers running the specified map (e.g. cs_italy)"""
        return Query("\\map\\", type=str)

    @property
    def ip(cls) -> Query[str]:
        """Fetches servers on the specified IP address.

        See Also
        --------
        :meth:`Client.fetch_server` for an query free version of this.
        """
        return Query("\\gameaddr\\", type=str)

    @property
    def running(cls) -> Query[Union[Game, int]]:
        """Fetches servers running a :class:`.Game` or an :class:`int` app id."""
        return Query("\\appid\\", type=(Game, int), callback=lambda game: getattr(game, "id", game))

    @property
    def not_running(cls) -> Query[Union[Game, int]]:
        """Fetches servers not running a :class:`.Game` or an :class:`int` app id."""
        return Query("\\nappid\\", type=(Game, int), callback=lambda game: getattr(game, "id", game))

    @property
    def match_tags(cls) -> Query[list[str]]:
        """Fetches servers with all of the given tag(s) in :attr:`GameServer.tags`."""
        return Query("\\gametype\\", type=list, callback=lambda items: f"[{','.join(items)}]")

    @property
    def match_hidden_tags(cls) -> Query[list[str]]:
        """Fetches servers with all of the given tag(s) in their 'hidden' tags only applies for :attr:`steam.LFD2`."""
        return Query("\\gamedata\\", type=list, callback=lambda items: f"[{','.join(items)}]")

    @property
    def match_any_hidden_tags(cls) -> Query[list[str]]:
        """Fetches servers with any of the given tag(s) in their 'hidden' tags only applies for :attr:`steam.LFD2`."""
        return Query("\\gamedataor\\", type=list, callback=lambda items: f"[{','.join(items)}]")

    @property
    def all(cls) -> QueryAll:
        """Fetches any servers. Any operations on this will fail."""
        return QueryAll()


class Query(Generic[T], metaclass=QueryMeta):
    r"""A :class:`pathlib.Path` like class for constructing Global Master Server queries.

    .. container:: operations

        .. describe:: x == y

            Checks if two queries are equal, order is checked.

        .. describe:: x / y

            Appends y's query to x.

        .. describe:: x | y

            Combines the two queries in ``\nor\[x\y]``.

        .. describe:: x & y

            Combines the two queries in ``\nand\[x\y]``.

    Examples
    --------
    .. code-block::

        >>> Query.running / TF2 / Query.not_empty / Query.secure
        <Query query='\\appid\\440\\empty\\1\\secure\\1'>
        >>> Query.not_empty / Query.not_full | Query.secure
        <Query query='\\empty\\1\\nor\\[\\full\\1\\secure\\1]'>
        >>> Query.name_match / "A cool server" | Query.match_tags / ["alltalk", "increased_maxplayers"]
        <Query query='\\nor\\[\\name_match\\A cool server\\gametype\\[alltalk,increased_maxplayers]]'>
    """

    # simple specification:
    # - immutable
    # - based on https://developer.valvesoftware.com/wiki/Master_Server_Query_Protocol

    __slots__ = ("_raw", "_type", "_callback")

    def __new__(
        cls,
        *raw: Union[Query, Operator, str],
        type: Union[type[T], tuple[type[T], ...], None] = None,
        callback: Optional[Callable[[T], str]] = lambda x: x,
    ) -> Query:
        self = super().__new__(cls)
        self._raw = raw
        self._type = type
        self._callback = callback
        return self

    def __repr__(self) -> str:
        return f"<Query query={self.query!r}>"

    def _process_op(self, other: T, op: Operator) -> Query[T]:
        cls = self.__class__

        if self._type and isinstance(other, self._type):
            return cls(self, op, other)

        if not isinstance(other, Query):
            return NotImplemented

        return cls(self, op, other, type=other._type, callback=other._callback)

    def __truediv__(self, other: T) -> Query[T]:
        return self._process_op(other, Operator.div)

    def __and__(self, other: T) -> Query[T]:
        # I'm not really sure what this does differently to __truediv__ or when to use it.
        return self._process_op(other, Operator.and_)

    def __or__(self, other: T) -> Query[T]:
        return self._process_op(other, Operator.or_)

    def __eq__(self, other: Query) -> bool:
        if not isinstance(other, Query):
            return NotImplemented
        return self._raw == other._raw

    @property
    def query(self) -> str:
        """:class:`str`: The actual query used for querying Global Master Servers."""

        if len(self._raw) == 1:  # string query
            return self._raw[0]

        # normal
        query_1, op, query_2 = self._raw

        return op.format(
            query_1.query,
            query_2.query if isinstance(query_2, Query) else query_1._callback(query_2),
        )


class ServerPlayer(NamedTuple):
    index: int
    name: str
    score: int
    play_time: timedelta


async def _handle_a2s_response(loop: asyncio.AbstractEventLoop, sock: socket.socket) -> bytes:
    packet = await loop.sock_recv(sock, 2048)
    header = struct.unpack_from("<l", packet)[0]

    if header == -1:  # single packet response
        return packet
    elif header == -2:  # multi packet response
        packets = [packet]
        payload_offset = -1

        # locate first packet and handle out of order packets
        while payload_offset == -1:
            # locate payload offset in uncompressed packet
            payload_offset = packet.find(b"\xff\xff\xff\xff", 0, 18)

            # locate payload offset in compressed packet
            if payload_offset == -1:
                payload_offset = packet.find(b"BZh", 0, 21)

            # if we still haven't found the offset receive the next packet
            if payload_offset == -1:
                packet = await loop.sock_recv(sock, 2048)
                packets.append(packet)

        # read header
        if payload_offset not in (10, 12, 18):  # Source
            raise ValueError(f"Unexpected payload_offset - {payload_offset}")
        packet_id, number_of_packets, packet_idx = struct.unpack_from("<LBB", packet, 4)
        compressed = (packet_id & 0x80000000) != 0
        if packet_idx != 0:
            raise ValueError("Unexpected first packet index")

        # receive any remaining packets
        for _ in range(number_of_packets - 1):
            packets.append(await loop.sock_recv(sock, 2048))

        packets = sorted({struct.unpack_from("<B", packet, 9)[0]: packet for packet in packets}.items())
        # reconstruct full response
        data = b"".join(x[1][payload_offset:] for x in packets)

        # decompress response if needed
        if compressed:
            size, check_sum = struct.unpack_from("<ll", packet, 10)
            data = decompress(data)

            if len(data) != size:
                raise ValueError(f"Response size mismatch - {len(data)} {size}")
            if check_sum != crc32(data):
                raise ValueError(f"Response check sum mismatch - {check_sum} {crc32(data)}")

        return data

    raise ValueError(f"Invalid response header - {header}")


class GameServer(SteamID):
    """Represents a game server.

    Attributes
    ----------
    name: :class:`str`
        The name of the server.
    game: :class:`.Game`
        The game of the server.
    ip: :class:`str`
        The ip of the server.
    port: :class:`int`
        The port of the server.
    tags: list[:class:`str`]
        The tags of the server.
    map: :class:`str`
        The map the server is running.
    bot_count: :class:`int`
        The number of bots in the server.
    player_count: :class:`int`
        The number of players the server.
    max_player_count: :class:`str`
        The maximum player count of the server.
    region: :class:`GameServerRegion`
        The region the server is in.
    version: :class:`str`
        The version of the server.
    """

    __slots__ = (
        "name",
        "game",
        "ip",
        "port",
        "tags",
        "map",
        "bot_count",
        "player_count",
        "max_player_count",
        "region",
        "version",
        "_secure",
        "_dedicated",
        "_loop",
    )

    def __init__(self, server: GameServerProto):
        super().__init__(server.steamid, type="GameServer")
        self.name = server.name
        self.game = Game(id=server.appid)
        self.ip = server.addr.split(":")[0]
        self.port = server.gameport
        self.tags = server.gametype.split(",")
        self.map = server.map
        self.bot_count = server.bots
        self.player_count = server.players
        self.max_player_count = server.max_players
        self.region = GameServerRegion.try_value(server.region)
        self.version = server.version

        self._secure = server.secure
        self._dedicated = server.dedicated
        self._loop = asyncio.get_event_loop()

    def __repr__(self) -> str:
        attrs = ("name", "game", "ip", "port", "region", "id", "type", "universe", "instance")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"<{self.__class__.__name__} {' '.join(resolved)}>"

    def __str__(self) -> str:
        return self.name

    def is_secure(self) -> bool:
        """:class:`bool`: Whether the sever is secured, likely with VAC."""
        return self._secure

    def is_dedicated(self) -> bool:
        """:class:`bool`: Whether the sever is dedicated."""
        return self._dedicated

    @asynccontextmanager
    async def connect(self, challenge: int, char: bytes) -> AbstractAsyncContextManager[StructIO]:
        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
            # steam uses TCP over UDP. I would use asyncio streams otherwise
            sock.setblocking(False)
            await self._loop.sock_connect(sock, (self.ip, self.port))

            sock.send(struct.pack("<lci", -1, char, challenge))
            data = await self._loop.sock_recv(sock, 512)
            _, header, challenge = struct.unpack_from("<lcl", data)

            if header != b"A":
                return

            sock.send(struct.pack("<lci", -1, char, challenge))
            try:
                data = await _handle_a2s_response(self._loop, sock)
            except ValueError:
                return

        yield StructIO(data)

    async def players(self, *, challenge: Literal[-1, 0] = 0) -> Optional[list[ServerPlayer]]:
        """|coro|
        Fetch a server's players.

        Parameters
        ----------
        challenge: :class:`int`
            The challenge for the request default is 0 can also be -1. You may need to change if the server doesn't seem
            to respond.

        Note
        ----
        It is recommended to use :func:`asyncio.wait_for` to allow this to return if the server doesn't respond.

        Returns
        -------
        Optional[list[:class:`ServerPlayer`]]
            The players, or ``None`` if something went wrong getting the info.

            ServerPlayer is a :class:`typing.NamedTuple` defined as:

            .. code-block:: python3

                class ServerPlayer(NamedTuple):
                    index: int
                    name: str
                    score: int
                    play_time: timedelta
        """
        async with self.connect(challenge, b"U") as buffer:
            header, number_of_players = buffer.read_struct("<4xcB")
            if header != b"D":
                return None

            return [
                ServerPlayer(
                    index=buffer.read_u8(),
                    name=buffer.read_cstring().decode("utf-8", "replace"),
                    score=buffer.read_long(),
                    play_time=timedelta(seconds=buffer.read_f32()),
                )
                for _ in range(number_of_players)
            ]

    async def rules(self, *, challenge: Literal[-1, 0] = 0) -> Optional[dict[str, str]]:
        """|coro|
        Fetch a server's rules (console variables). e.g. ``sv_gravity`` or ``sv_voiceenable``.

        Parameters
        ----------
        challenge: :class:`int`
            The challenge for the request default is 0 can also be -1. You may need to change if the server doesn't seem
            to respond.

        Note
        ----
        It is recommended to use :func:`asyncio.wait_for` to allow this to return if the server doesn't respond.

        Returns
        -------
        Optional[dict[:class:`str`, :class:`str`]]
            The server's rules.
        """
        async with self.connect(challenge, b"V") as buffer:
            header, number_of_rules = buffer.read_struct("<4xcH")
            if header != b"E":
                return None

            return {
                buffer.read_cstring().decode("utf-8", "replace"): buffer.read_cstring().decode("utf-8", "replace")
                for _ in range(number_of_rules)
            }



"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import itertools
import math
import re
from collections import deque
from collections.abc import Callable, Coroutine
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Any, Generic, Optional, TypeVar, Union

from bs4 import BeautifulSoup
from typing_extensions import TypeAlias

from . import utils
from .comment import Comment
from .enums import TradeOfferState

if TYPE_CHECKING:
    from .abc import BaseUser, Channel, Message
    from .channel import ClanChannel, ClanMessage, DMChannel, GroupChannel, GroupMessage
    from .clan import Clan
    from .state import ConnectionState
    from .trade import DescriptionDict, TradeOffer


T = TypeVar("T")
C = TypeVar("C", bound="Channel")
M = TypeVar("M", bound="Message")
MaybeCoro: TypeAlias = "Callable[[T], Union[bool, Coroutine[Any, Any, bool]]]"
UNIX_EPOCH = datetime.utcfromtimestamp(0)


class AsyncIterator(Generic[T]):
    """A class from which async iterators (see :pep:`525`) can ben easily derived.

    .. container:: operations

        .. describe:: async for y in x

            Iterates over the contents of the async iterator.

    Attributes
    ----------
    before: :class:`datetime.datetime`
        When to find objects before.
    after: :class:`datetime.datetime`
        When to find objects after.
    limit: Optional[:class:`int`]
        The maximum size of the :attr:`AsyncIterator.queue`.
    queue: :class:`collections.deque[T]`
        The queue containing the elements of the iterator.
    """

    __slots__ = ("before", "after", "limit", "queue", "_is_filled", "_state")

    def __init__(
        self, state: ConnectionState, limit: Optional[int], before: Optional[datetime], after: Optional[datetime]
    ):
        self._state = state
        self.before = before or (datetime.utcnow() + timedelta(seconds=30))  # dont run into race condition hell
        self.after = after or UNIX_EPOCH
        self._is_filled = False
        self.queue: deque[T] = deque()
        self.limit = limit

    def append(self, element: T) -> bool:
        if self.limit is None:
            self.queue.append(element)
            return True
        if len(self.queue) < self.limit:
            self.queue.append(element)
            return True
        if len(self.queue) == self.limit:
            self.queue.append(element)

        return False

    def get(self, **attrs: Any) -> Coroutine[None, None, Optional[T]]:
        """|coro|
        A helper function which is similar to :func:`~steam.utils.get` except it runs over the :class:`AsyncIterator`.

        This is roughly equipment to: ::

            elements = await AsyncIterator.flatten()
            element = steam.utils.get(name='Item', elements)

        Example
        -------
        Getting the last comment from a user named 'Dave' or None: ::

            msg = await User.comments().get(author__name='Dave')

        Parameters
        ----------
        **attrs
            Keyword arguments that denote attributes to match.

        Returns
        -------
        Optional[T]
            The first element from the ``iterable`` which matches all the traits passed in ``attrs`` or ``None`` if no
            matching element was found.
        """

        def predicate(elem: T) -> bool:
            for attr, val in attrs.items():
                nested = attr.split("__")
                obj = elem
                for attribute in nested:
                    obj = getattr(obj, attribute)

                if obj != val:
                    return False
            return True

        return self.find(predicate)

    async def find(self, predicate: MaybeCoro[T]) -> Optional[T]:
        """|coro|
        A helper function which is similar to :func:`~steam.utils.find` except it runs over the :class:`AsyncIterator`.
        However unlike :func:`~steam.utils.find`, the predicate provided can be a |coroutine_link|_.

        This is roughly equivalent to: ::

            elements = await AsyncIterator.flatten()
            element = steam.utils.find(elements, lambda e: e.name == 'Item')

        Example
        -------
        Getting the last trade with a message or None: ::

            def predicate(trade: steam.TradeOffer) -> bool:
                return trade.message is not None

            trade = await Client.trade_history().find(predicate)

        Parameters
        ----------
        predicate: Callable[[T], Union[:class:`bool`, Awaitable[:class:`bool`]]]
            A callable/coroutine that returns a boolean.

        Returns
        -------
        Optional[T]
            The first element from the iterator for which the ``predicate`` returns ``True`` or ``None`` if no matching
            element was found.
        """
        async for elem in self:
            ret = await utils.maybe_coroutine(predicate, elem)
            if ret:
                return elem

    async def flatten(self) -> list[T]:
        """|coro|
        A helper function that iterates over the :class:`AsyncIterator` returning a list of all the elements in the
        iterator.

        This is equivalent to: ::

            elements = [element async for element in AsyncIterator]

        Returns
        -------
        list[T]
            A list of every element in the iterator.
        """
        return [element async for element in self]

    def __aiter__(self) -> AsyncIterator[T]:
        return self

    def __anext__(self) -> Coroutine[None, None, T]:
        return self.next()

    async def next(self) -> T:
        """|coro|
        Advances the iterator by one, if possible.

        Raises
        --------
        :exc:`StopAsyncIteration`
            There are no more elements in the iterator.
        """
        if not self.queue:
            if self._is_filled:
                raise StopAsyncIteration
            await self.fill()
            self._is_filled = True
        if not self.queue:  # yikes
            raise StopAsyncIteration
        return self.queue.pop()

    async def fill(self) -> None:
        raise NotImplementedError


class CommentsIterator(AsyncIterator[Comment]):
    __slots__ = ("owner",)

    def __init__(
        self,
        state: ConnectionState,
        limit: Optional[int],
        before: Optional[datetime],
        after: Optional[datetime],
        owner: Union[BaseUser, Clan],
    ):
        super().__init__(state, limit, before, after)
        self.owner = owner

    async def fill(self) -> None:
        data = await self._state.http.get_comments(
            id64=self.owner.id64, comment_path=self.owner.comment_path, limit=self.limit
        )
        soup = BeautifulSoup(data["comments_html"], "html.parser")
        to_fetch = []

        after_timestamp = self.after.timestamp()
        before_timestamp = self.before.timestamp()

        for comment in soup.find_all("div", attrs={"class": "commentthread_comment responsive_body_text"}):
            timestamp = int(comment.find("span", attrs={"class": "commentthread_comment_timestamp"})["data-timestamp"])
            if after_timestamp < timestamp < before_timestamp:
                author_id = int(comment.find("a", attrs={"class": "commentthread_author_link"})["data-miniprofile"])
                comment_id = int(re.findall(r"comment_([0-9]*)", str(comment))[0])
                content = comment.find("div", attrs={"class": "commentthread_comment_text"}).get_text().strip()
                to_fetch.append(utils.make_id64(author_id))
                comment = Comment(
                    state=self._state,
                    id=comment_id,
                    timestamp=datetime.utcfromtimestamp(timestamp),
                    content=content,
                    author=author_id,
                    owner=self.owner,
                )
                if not self.append(comment):
                    return
        users = await self._state.fetch_users(to_fetch)
        for user, comment in itertools.product(users, self.queue):
            if comment.author == user.id:
                comment.author = user


class TradesIterator(AsyncIterator["TradeOffer"]):
    __slots__ = ("_active_only",)

    def __init__(
        self,
        state: ConnectionState,
        limit: Optional[int],
        before: Optional[datetime],
        after: Optional[datetime],
        active_only: bool,
    ):
        super().__init__(state, limit, before, after)
        self._active_only = active_only

    async def fill(self) -> None:
        from .trade import TradeOffer

        resp = await self._state.http.get_trade_history(100, None)
        resp = resp["response"]
        total = resp.get("total_trades", 0)
        if not total:
            return

        users_to_fetch = []
        descriptions = resp.get("descriptions", [])
        after_timestamp = self.after.timestamp()
        before_timestamp = self.before.timestamp()

        async def process_trade(data: dict, descriptions: DescriptionDict) -> None:
            if not after_timestamp < data["time_init"] < before_timestamp:
                return
            for item in descriptions:
                for asset in data.get("assets_received", []):
                    if item["classid"] == asset["classid"] and item["instanceid"] == asset["instanceid"]:
                        asset.update(item)
                for asset in data.get("assets_given", []):
                    if item["classid"] == asset["classid"] and item["instanceid"] == asset["instanceid"]:
                        asset.update(item)

            # patch in the attributes cause steam is cool
            data["tradeofferid"] = data["tradeid"]
            data["accountid_other"] = data["steamid_other"]
            data["trade_offer_state"] = data["status"]
            data["items_to_give"] = data.get("assets_given", [])
            data["items_to_receive"] = data.get("assets_received", [])

            trade = await TradeOffer._from_api(state=self._state, data=data)
            users_to_fetch.append(trade.partner)

            if not self._active_only:
                if not self.append(trade):
                    raise StopAsyncIteration
            elif trade.state in (
                TradeOfferState.Active,
                TradeOfferState.ConfirmationNeed,
            ):
                if not self.append(trade):
                    raise StopAsyncIteration

        try:
            for trade in resp.get("trades", []):
                await process_trade(trade, descriptions)

            previous_time = trade["time_init"]
            users = await self._state.fetch_users(users_to_fetch)
            for user, trade in itertools.product(users, self.queue):
                if trade.partner == user.id64:
                    trade.partner = user

            if total < 100:
                for page in range(200, math.ceil((total + 100) / 100) * 100, 100):
                    users_to_fetch = []
                    resp = await self._state.http.get_trade_history(page, previous_time)
                    resp = resp["response"]

                    for trade in resp.get("trades", []):
                        previous_time = trade["time_init"]
                        await process_trade(trade, descriptions)
                    users = await self._state.fetch_users(users_to_fetch)
                    for user, trade in itertools.product(users, self.queue):
                        if trade.partner == user.id64:
                            trade.partner = user
        except StopAsyncIteration:
            users = await self._state.fetch_users(users_to_fetch)  # fetch the final users
            for user, trade in itertools.product(users, self.queue):
                if trade.partner == user.id64:
                    trade.partner = user


class ChannelHistoryIterator(AsyncIterator[M], Generic[M, C]):
    __slots__ = ("channel", "_actual_before")

    def __init__(
        self,
        channel: C,
        state: ConnectionState,
        limit: Optional[int],
        before: Optional[datetime],
        after: Optional[datetime],
    ):
        super().__init__(state, limit, before, after)
        self.before = before or UNIX_EPOCH
        self._actual_before = before or datetime.utcnow()  # steam doesn't know how timestamps work with this endpoint
        self.channel = channel


class DMChannelHistoryIterator(ChannelHistoryIterator["UserMessage", "DMChannel"]):
    __slots__ = ("participant",)

    def __init__(
        self,
        channel: DMChannel,
        state: ConnectionState,
        limit: Optional[int],
        before: Optional[datetime],
        after: Optional[datetime],
    ):
        super().__init__(channel, state, limit, before, after)
        self.participant = channel.participant

    async def fill(self) -> None:
        from .message import UserMessage

        after_timestamp = int(self.after.timestamp())
        before_timestamp = int(self.before.timestamp())
        actual_before_timestamp = int(self._actual_before.timestamp())

        last_message_timestamp = before_timestamp

        while True:
            msgs = await self._state.fetch_user_history(
                self.participant.id64, start=after_timestamp, last=last_message_timestamp
            )
            if msgs is None:
                return

            for message in msgs.body.messages:
                if not (after_timestamp < message.timestamp <= actual_before_timestamp):
                    return

                new_message = UserMessage.__new__(UserMessage)
                Message.__init__(new_message, channel=self.channel, proto=message)
                new_message.author = (
                    self.participant if message.accountid == self.participant.id else self._state.client.user
                )
                new_message.created_at = datetime.utcfromtimestamp(message.timestamp)
                if not self.append(new_message):
                    return

            last_message_timestamp = int(message.timestamp)

            if not msgs.body.more_available:
                return


class GroupChannelHistoryIterator(
    ChannelHistoryIterator[Union["ClanMessage", "GroupMessage"], Union["ClanChannel", "GroupChannel"]]
):

    __slots__ = ("group",)

    def __init__(
        self,
        channel: Union[ClanChannel, GroupChannel],
        state: ConnectionState,
        limit: Optional[int],
        before: Optional[datetime],
        after: Optional[datetime],
    ):
        super().__init__(channel, state, limit, before, after)
        self.group = channel.group or channel.clan

    async def fill(self) -> None:
        from .message import ClanMessage, GroupMessage

        after_timestamp = int(self.after.timestamp())
        before_timestamp = int(self.before.timestamp())
        actual_before_timestamp = int(self._actual_before.timestamp())
        last_message_timestamp = before_timestamp
        group_id = getattr(self.group, "chat_id", None) or self.group.id

        while True:
            msgs = await self._state.fetch_group_history(
                group_id, self.channel.id, start=after_timestamp, last=last_message_timestamp
            )
            if msgs is None:
                return

            to_fetch = []

            for message in msgs.body.messages:
                if not (after_timestamp < message.server_timestamp <= actual_before_timestamp):
                    return

                new_message = (
                    GroupMessage.__new__(GroupMessage) if self.channel.group else ClanMessage.__new__(ClanMessage)
                )
                Message.__init__(new_message, channel=self.channel, proto=message)
                new_message.author = utils.make_id64(message.sender)
                to_fetch.append(new_message.author)
                new_message.created_at = datetime.utcfromtimestamp(message.server_timestamp)
                if not self.append(new_message):
                    users = await self._state.fetch_users(to_fetch)

                    for user, message in itertools.product(users, self.queue):
                        if message.author == user.id64:
                            message.author = user

                    return

            last_message_timestamp = int(message.server_timestamp)

            users = await self._state.fetch_users(to_fetch)
            for user, message in itertools.product(users, self.queue):
                if message.author == user.id64:
                    message.author = user

            if not msgs.body.more_available:
                return
